/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


/*
 * If instant apply (browser.preferences.instantApply) is set
 * (default for mac & linux), preferences are applied instantly
 * when changed in ui and dialog has only one button - close.
 * But I couldn't get the preference automagic to work with trees,
 * so rootFolders are persisted by hand.
 *
 * (Also, it seems that the close button's dlgtype is "cancel",
 * which a bit weird, imho. Basically, ondialogaccept is not fired 
 * when dialog is closed.)
 */

var savefileto_options =
{
    doOnLoad: function()
    {
        this.rootFolders = {};
        var types = ["page", "link", "image"];
        for (var i = 0; i < types.length; i++)
        {
            var tree = document.getElementById("tree_" + types[i] + "_rootFolders");
            var view = new savefileto_RootFolderTreeView(types[i]);
            tree.view = view;
            // view model cannot be accessed via tree.view.model, so we must store it elsewhere
            this.rootFolders[types[i]] = view.data;
            // update ui
            this.updateButtonStates(types[i]);
        }
    },
    
    doOnSelectFolder: function(type)
    {
        this.updateButtonStates(type);
    },
    
    addFolder: function(type)
    {
        var result = this.openFolderDialog();
        if (! result)
            return;
        var tree = document.getElementById("tree_" + type + "_rootFolders");
        this.rootFolders[type].push(result);
        var rowIndex = tree.view.rowCount - 1;
        tree.treeBoxObject.rowCountChanged(rowIndex, 1);
        tree.treeBoxObject.ensureRowIsVisible(rowIndex);
        tree.view.selection.select(rowIndex);
        // XXX: instant-apply-by-hand
        if (savefileto_util.isInstantApply())
            savefileto_options.persistRootFolders(type);
    },
    
    editFolder: function(type)
    {
        var tree = document.getElementById("tree_" + type + "_rootFolders");
        var rowIndex = tree.currentIndex;
        tree.treeBoxObject.ensureRowIsVisible(rowIndex);
        var result = this.openFolderDialog(this.rootFolders[type][rowIndex]);
        if (! result)
            return;
        this.rootFolders[type][rowIndex] = result;
        tree.treeBoxObject.invalidateRow(rowIndex);
        // XXX: instant-apply-by-hand
        if (savefileto_util.isInstantApply())
            savefileto_options.persistRootFolders(type);
    },
    
    removeFolder: function(type)
    {
        var tree = document.getElementById("tree_" + type + "_rootFolders");
        var rowIndex = tree.currentIndex;
        this.rootFolders[type].splice(rowIndex, 1);
        tree.treeBoxObject.rowCountChanged(rowIndex, -1);
        // XXX: instant-apply-by-hand
        if (savefileto_util.isInstantApply())
            savefileto_options.persistRootFolders(type);
    },
    
    moveFolder: function(type, delta)
    {
        var tree = document.getElementById("tree_" + type + "_rootFolders");
        var rowIndex = tree.currentIndex;
        var folder = this.rootFolders[type].splice(rowIndex, 1)[0];
        this.rootFolders[type].splice(rowIndex + delta, 0, folder);
        tree.treeBoxObject.invalidateRow(rowIndex);
        tree.treeBoxObject.invalidateRow(rowIndex + delta);
        tree.treeBoxObject.ensureRowIsVisible(rowIndex + delta);
        tree.view.selection.select(rowIndex + delta);
        // XXX: instant-apply-by-hand
        if (savefileto_util.isInstantApply())
            savefileto_options.persistRootFolders(type);
    },
    
    updateButtonStates: function(type)
    {
        var tree = document.getElementById("tree_" + type + "_rootFolders");
        var selection = tree.view.selection;
        var isSelected = (selection.count > 0);
        var isFirst = (isSelected && (tree.currentIndex == 0));
        var isLast = (isSelected && (tree.currentIndex == (tree.view.rowCount - 1)));
        document.getElementById("btn_" + type + "_editFolder").setAttribute("disabled", (! isSelected));
        document.getElementById("btn_" + type + "_removeFolder").setAttribute("disabled", (! isSelected));
        document.getElementById("btn_" + type + "_moveFolderUp").setAttribute("disabled", ((! isSelected) || isFirst));
        document.getElementById("btn_" + type + "_moveFolderDown").setAttribute("disabled", ((! isSelected) || isLast));
    },
    
    openFolderDialog: function(folder)
    {
        if (folder === undefined)
            folder = {name: "", path: ""};
        var params = {inn: folder, out: null}
        document.documentElement.openSubDialog("chrome://savefileto/content/folder.xul", null, params);
        return params.out;
    },
    
    // XXX: couldm't get the automagic preference thingy to work with trees so we do it by hand
    persistRootFolders: function(type)
    {
        var prefs = savefileto_util.getPrefBranch("extensions.savefileto.");
        var s = savefileto_util.foldersToString(this.rootFolders[type]);
        savefileto_util.setUnicharPref(prefs, type + ".rootFolders", s);
    },
    
    doOnDialogAccept: function()
    {
        // XXX: non-instant-apply-by-hand
        this.persistRootFolders("page");
        this.persistRootFolders("link");
        this.persistRootFolders("image");
    }
}


function savefileto_RootFolderTreeView(type)
{
    var prefs = savefileto_util.getPrefBranch("extensions.savefileto.");
    var rootFolders = savefileto_util.getUnicharPref(prefs, type + ".rootFolders");
    this.data = savefileto_util.parseFolders(rootFolders, false);
}

savefileto_RootFolderTreeView.prototype =
{
    get rowCount() { return this.data.length; },
    
    setTree: function(treeBox)
    {
        this.treeBox = treeBox;
    },
    
    getCellText: function(row, col)
    {
        // a bit shaky code, but column ids ends with "_name" and "_path"  :)
        var index = col.id.lastIndexOf("_");
        col = col.id.substr(index + 1);
        return this.data[row][col];
    },
    
    isContainer: function(row) { return false; },
    isSeparator: function(row) { return false; },
    isSorted: function() { return false; },
    getLevel: function(row) { return 0; },
    getImageSrc: function(row, col) { return null; },
    getRowProperties: function(row, props) {},
    getCellProperties: function(row, col, props) {},
    getColumnProperties: function(colid, col, props) {}
}
